; Interrupt vector table
dword_0:        dc.l 0
                dc.l init

; Attributes: noreturn

init:
                lea     $A00000,a0      ; Load ODIE/whatever register base
                move.b  #8,2(a0)        ; Switch host interface to HOST mode

wait_init:
                btst    #1,2(a0)        ; Is TxRDY set?
                beq.s   wait_init
                move.b  #$FF,3(a0)      ; Write host interface data
                move.b  #$10,($800).w
                move.b  (dword_0).w,d0  ; Zero out D0
                andi.b  #$FC,$1B(a0)    ; Clear sound memory bank select
                ori.b   #2,$1B(a0)      ; Select sound memory bank 2
                lea     $400001,a1      ; Sample RAM base perhaps??
                move.b  #$55,(a1)       ; Write to memory
                move.b  (a1),d1         ; Read from memory again
                cmpi.b  #$55,d1         ; Did the value change?
                bne.s   not_ram
                bset    #5,d0           ; Set a flag in bit 5

not_ram:
                andi.b  #$FC,$1B(a0)    ; Clear sound memory bank select
                ori.b   #1,$1B(a0)      ; Select sound memory bank 1
                lea     $400024,a1
                cmpi.b  #$FD,(a1)
                bne.s   not_old_s2k
                ori.b   #0,d0           ; Model 0 (soundscape.co0)
                                        ; Soundscape S-2000 with old 2M ROM
                bra.s   wait_done
; ---------------------------------------------------------------------------

not_old_s2k:
                cmpi.b  #$F,(a1)
                bne.s   not_new_s2k
                ori.b   #1,d0           ; Model 1 (soundscape.co1)
                                        ; Soundscape S-2000 with new 1M2 ROM
                bra.s   wait_done
; ---------------------------------------------------------------------------

not_new_s2k:
                cmpi.b  #$19,(a1)
                bne.s   not_elite
                ori.b   #2,d0           ; Model 2 (soundscape.co2)
                                        ; Soundscape Elite
                bra.s   wait_done
; ---------------------------------------------------------------------------

not_elite:
                cmpi.b  #0,(a1)         ; If it's not zero, we don't know what this is
                bne.s   cannot_id
                move.w  #$5000,$26(a0)  ; Something newer/other than ODIE?
                cmpi.b  #$50,6(a0)      ; Test interrupt control register
                beq.s   not_pnp_opus
                ori.b   #3,d0           ; Model 3 (soundscape.co3)
                                        ; Soundscape PnP with OPUS
                bra.s   wait_done
; ---------------------------------------------------------------------------

not_pnp_opus:
                ori.b   #4,d0           ; Model 4 (soundscape.co4)
                                        ; Soundscape PnP with OTTO
                bra.s   wait_done
; ---------------------------------------------------------------------------

cannot_id:
                ori.b   #$F,d0

wait_done:
                btst    #1,2(a0)
                beq.s   wait_done
                move.b  #$FE,3(a0)      ; Write host interface data

wait_report:
                btst    #1,2(a0)
                beq.s   wait_report
                move.b  d0,3(a0)        ; Write D0 contents to host interface

                move.b  #$25,8(a0)      ; Enable normal DMA A to host
                move.w  #$40,$A(a0)     ; Set bits 16-22 of DMA A buffer address
                move.w  #01,$C(a0)      ; Set bits 0-15 of DMA A buffer address

wait_dma:
                btst    #1,2(a0)
                beq.s   wait_dma
                move.b  #$FD,3(a0)      ; Write host interface data

                lea     $400000,a1      ; Load ROM base address

next_byte:
                move.b  (a1)+,d0        ; Read byte from ROM
wait_byte:
                btst    #1,2(a0)
                beq.s   wait_byte
                move.b  d0,3(a0)        ; Write host interface data
                bra.s   next_byte

hang_self:
                bra.s   hang_self

; ---------------------------------------------------------------------------
                nop
                nop
; ---------------------------------------------------------------------------
                dc.b   0
                dc.b   0

                END     init
